# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""API functions of the pure connector."""

from typing import Callable

from .types import PureConfigs, PureRecord
from .utils import download_file, get_research_output_count, get_research_outputs


def import_from_pure(
    import_func: Callable, pure_record: PureRecord, configs: PureConfigs
):
    """Import record from pure."""
    return import_func(pure_record, configs, download_file)


def pure_records(configs):
    """Generator to yield records from pure."""
    research_count = get_research_output_count(configs.endpoint, configs.token)
    size = 100
    offset = 0

    while research_count > 0:
        research_output = get_research_outputs(
            configs.endpoint, configs.token, size, offset
        )

        for record in research_output:
            yield record

        offset += size
        research_count -= size



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Technische Universität Graz
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Click options."""

from json import JSONDecodeError, loads
from os.path import dirname, isabs, isfile, join

from click import ParamType, secho


def load_file_as_string(path):
    """Open a file and return the content as UTF-8 encoded string."""
    if not isabs(path):
        path = join(dirname(__file__), path)

    if not isfile(path):
        return ""

    with open(path, "rb") as file_pointer:
        content = file_pointer.read()
        return content.decode("utf-8")


class JSON(ParamType):
    """JSON provides the ability to load a json from a string or a file."""

    name = "JSON"

    def convert(self, value, param, ctx):
        """This method converts the json-str or json-file to the dictionary representation."""
        if isfile(value):
            value = load_file_as_string(value)

        try:
            return loads(value)
        except JSONDecodeError:
            secho("ERROR - Invalid JSON provided.", fg="red")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Technische Universität Graz
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Scheduled tasks for celery."""

from typing import Callable

from celery import shared_task
from flask import current_app
from flask_mail import Message

from .api import import_from_pure, pure_records
from .types import PureConfigs


def config_variables() -> tuple[Callable, Callable, PureConfigs]:
    """Configuration variables."""
    import_func = current_app.config["PURE_IMPORT_FUNC"]
    sieve_func = current_app.config["PURE_SIEVE_FUNC"]
    endpoint = current_app.config["PURE_PURE_ENDPOINT"]
    token = current_app.config["PURE_PURE_TOKEN"]
    pure_username = current_app.config["PURE_PURE_USERNAME"]
    pure_password = current_app.config["PURE_PURE_PASSWORD"]
    user_email = current_app.config["PURE_USER_EMAIL"]
    recipients = current_app.config["PURE_ERROR_MAIL_RECIPIENTS"]
    sender = current_app.config["PURE_ERROR_MAIL_SENDER"]

    pure_configs = PureConfigs(
        endpoint, token, pure_username, pure_password, user_email, recipients, sender
    )

    return import_func, sieve_func, pure_configs


@shared_task(ignore_result=True)
def import_records_from_pure():
    """Import records from pure."""
    import_func, sieve_func, configs = config_variables()

    for pure_record in pure_records(configs):
        if not sieve_func(pure_record):
            continue

        try:
            import_from_pure(import_func, pure_record, configs)
        except RuntimeError:
            msg = Message(
                "ERROR: importing from pure",
                sender=configs.sender,
                recipients=configs.recipients,
                body="record id",
            )
            current_app.extensions["mail"].send(msg)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Technische Universität Graz
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Utility methods."""

from json import loads
from shutil import copyfileobj

from requests import get
from requests.auth import HTTPBasicAuth

from .types import URL, FilePath, PureID


def headers(pure_api_key: str) -> dict[str, str]:
    """Headers."""
    return {
        "api-key": pure_api_key,
        "accept": "application/json",
    }


def get_research_output_count(pure_api_key: str, pure_api_url: str) -> int:
    """Get the amount of available research outputs at /research-outputs endpoint.

    Return -1 on failure.
    """
    url = f"{pure_api_url}/research-outputs"
    response = get(url, headers=headers(pure_api_key), timeout=10)

    if response.status_code != 200:
        return -1

    return int(loads(response.text)["count"])


def get_research_outputs(
    pure_api_key: str, pure_api_url: str, size: int, offset: int
) -> list[dict]:
    """Get a list of research outputs.

    Pure API identifies a series by the following parameters:
    The *size* parameter defines the length of the series.
    The *offset* parameter defines the offset of the series.
    Return [] if the GET request is not OK.
    """
    url = f"{pure_api_url}/research-outputs?size={size}&offset={offset}"
    response = get(url, headers=headers(pure_api_key), timeout=10)

    if response.status_code != 200:
        return []

    response_json = loads(response.text)
    items = response_json["items"]
    return items


def store_file_temporarily(file_url: URL, file_path: FilePath, auth: HTTPBasicAuth):
    """Download file."""
    with get(file_url, stream=True, auth=auth, timeout=10) as response:
        with open(file_path, "wb") as file_pointer:
            copyfileobj(response.raw, file_pointer)


def download_file(
    pure_id: PureID,
    file_url: URL,
    pure_username: str,
    pure_password: str,
) -> str:
    """Download a file from Pure to given destination path with given file name.

    Return path to the downloaded file upon success, empty string upon failure.
    """
    file_path = f"/tmp/{pure_id}.pdf"
    auth = HTTPBasicAuth(pure_username, pure_password)
    store_file_temporarily(file_url, file_path, auth)

    return file_path



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Technische Universität Graz.
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module that adds pure."""


class InvenioPure:
    """invenio-pure extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        app.extensions["invenio-pure"] = self



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Technische Universität Graz.
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module that adds pure."""
from typing import Callable

from .types import (
    URL,
    EmailAddress,
    FilePath,
    PureConfigs,
    PureID,
    PurePassword,
    PureRecord,
    PureToken,
    PureUsername,
)

PURE_CELERY_BEAT_SCHEDULE = {}
"""The celery beat schedule is used to configure the import schedule.

Following is a example configuration. One option to add this to the
global celery beat schedule is to update the CELERY_BEAT_SCHEDULE
dict in invenio.cfg by the example below:

.. code-block:: python

    "synchronize_records": {
        "task": "invenio_pure.tasks.import_records_from_pure",
        "schedule": crontab(hour=1, minute=0, day_of_week=0),
    }
"""

PURE_IMPORT_FUNC: Callable[
    [
        PureRecord,
        PureConfigs,
        Callable[[PureID, URL, PureUsername, PurePassword], FilePath],
    ],
    None,
] = None
"""This function is called to import the pure record into the repository.

It needs as an parameter the pure record as a json, the pure configuration
and a callable which is provided from this package to download the file
from pure."""

PURE_SIEVE_FUNC: Callable[[PureRecord], bool] = None
"""This function implements the import criteria.

The import criteria handles the conditions which have to be true
as that the record will be imported into the repository.
"""

PURE_PURE_ENDPOINT: URL = ""
"""This is the endpoint of the pure instance."""

PURE_PURE_TOKEN: PureToken = ""
"""This is the token to be allowed to use the API."""

PURE_PURE_USERNAME: PureUsername = ""
"""This is the pure username which is necessary to download files."""

PURE_PURE_PASSWORD: PurePassword = ""
"""This is the pure password which is necessary to download files."""

PURE_USER_EMAIL: EmailAddress = ""
"""This is the user email of the pure user within the InvenioRDM instance."""

PURE_ERROR_MAIL_RECIPIENTS: list[EmailAddress] = []
"""The list of recipients to send emails if errors happen in the import process."""

PURE_ERROR_MAIL_SENDER: EmailAddress = ""
"""The sender of the error emails."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 Technische Universität Graz.
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module that adds pure."""

from .errors import PureAPIError, PureRuntimeError
from .ext import InvenioPure
from .types import URL, PureConfigs, PureID, PureRecord

__version__ = "0.1.2"

__all__ = (
    "__version__",
    "InvenioPure",
    "PureConfigs",
    "PureRecord",
    "PureID",
    "PureAPIError",
    "PureRuntimeError",
    "URL",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2022 Technische Universität Graz
#
# invenio-rdm-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module that adds pure."""

from setuptools import setup

setup()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 Graz University of Technology.
#
# invenio-pure is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""CLI commands for Invenio-RDM-Pure."""


from click import STRING, group, option
from click_params import URL
from flask import current_app
from flask.cli import with_appcontext

from .api import import_from_pure
from .click_options import JSON
from .types import PureConfigs


@group()
def pure():
    """Commands for InvenioRdmPure."""


@pure.command("import")
@with_appcontext
@option("--endpoint", type=URL)
@option("--token", type=STRING)
@option("--pure_username", type=STRING)
@option("--pure_password", type=STRING)
@option("--user-email", type=STRING)
@option("--pure-record", type=JSON())
def import_records_from_pure(
    endpoint, token, pure_username, pure_password, user_email, pure_record
):
    """Import a record from given JSON file or JSON string."""
    import_func = current_app.config["PURE_IMPORT_FUNC"]
    recipients = current_app.config["PURE_ERROR_MAIL_RECIPIENTS"]
    sender = current_app.config["PURE_ERROR_MAIL_SENDER"]
    configs = PureConfigs(
        endpoint, token, pure_username, pure_password, user_email, recipients, sender
    )
    record = import_from_pure(import_func, pure_record, configs)
    print(f"record.id: {record.id}")
